﻿
-- declard color codes for console messages
local RED     = "|cffff0000"
local GREEN   = "|cff00ff00"
local BLUE    = "|cff0000ff"
local MAGENTA = "|cffff00ff"
local YELLOW  = "|cffffff00"
local CYAN    = "|cff00ffff"
local WHITE   = "|cffffffff"
local ORANGE  = "|cffffba00"

NauticusFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceEvent-2.0", "AceDB-2.0")

local Nauticus = Nauticus

local L = AceLibrary("AceLocale-2.2"):new("Nauticus")

local platforms, transports, transitData =
	Nauticus.platforms, Nauticus.transports, Nauticus.transitData

-- constants
local ARTWORKPATH = "Interface\\AddOns\\Nauticus\\Artwork\\"

NauticusFu:RegisterDB("NauticusDB", "NauticusDBPC")
NauticusFu:RegisterDefaults("profile", {
	factionSpecific = true,
	zoneSpecific = false,
	cityAlias = true;
} )

function NauticusFu:OnInitialize()
	self.hasIcon = true
	self.title = "Nauticus"
	self:SetIcon(ARTWORKPATH.."NauticusLogo")
end

function NauticusFu:IsFactionSpecific()
    return self.db.profile.factionSpecific
end

function NauticusFu:ToggleFaction()
    self.db.profile.factionSpecific = not self.db.profile.factionSpecific
    self:Update()
end

function NauticusFu:IsZoneSpecific()
    return self.db.profile.zoneSpecific
end

function NauticusFu:ToggleZone()
    self.db.profile.zoneSpecific = not self.db.profile.zoneSpecific
    self:Update()
end

function NauticusFu:IsAlias()
    return self.db.profile.cityAlias
end

function NauticusFu:ToggleAlias()
    self.db.profile.cityAlias = not self.db.profile.cityAlias
    self:Update()
end

local tablet = AceLibrary("Tablet-2.0")
function NauticusFu:OnTooltipUpdate()

	local cat = tablet:AddCategory(
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0
	)

	if Nauticus.activeTransit ~= -1 and Nauticus:GetKnownData(Nauticus.activeTransit) ~= nil then
		local transit = Nauticus.activeTransit

		cat:AddLine('text', Nauticus.activeTransitName)

		for index, data in pairs(platforms[transit]) do
			local platname

			if self:IsAlias() then
				platname = data.alias
			else
				platname = data.name
			end

			local cat = tablet:AddCategory(
				'text', platname,
				'columns', 2,
				'child_textR', 1,
				'child_textG', 1,
				'child_textB', 0
			)

			local the_time = Nauticus.activeData[index]
			local depOrArr
			local r,g,b

			if the_time < 0 then
				the_time = -the_time

				if the_time > 30 then
					r,g,b = 1,1,0
				else
					r,g,b = 1,0,0
				end

				depOrArr = L["Departure"]
			else
				r,g,b = 0,1,0
				depOrArr = L["Arrival"]
			end

			local formatted_time

			if the_time >= 60 then
				formatted_time = format("%dm %02ds", the_time/60, math.fmod(the_time, 60) )
			else
				formatted_time = format("%ds", the_time)
			end

			cat:AddLine('text', depOrArr..":",
				'text2', formatted_time,
				'text2R', r, 'text2G', g, 'text2B', b)

		end

		if Nauticus.debug then
			cat:AddLine('text', "Age:",
				'text2', SecondsToTime(GetTime()-Nauticus:GetKnownData(Nauticus.activeTransit)))
		end

	elseif Nauticus.activeTransit ~= -1 and Nauticus:GetKnownData(Nauticus.activeTransit) == nil then
		local transit = Nauticus.activeTransit

		cat:AddLine('text', Nauticus.activeTransitName)

		for index, data in pairs(platforms[transit]) do
			local platname

			if self:IsAlias() then
				platname = data.alias
			else
				platname = data.name
			end

			local cat = tablet:AddCategory(
				'text', platname,
				'columns', 2,
				'child_textR', 1,
				'child_textG', 1,
				'child_textB', 0
			)

			cat:AddLine('text', L["Not Available"])
		end

	elseif Nauticus.activeTransit == -1 then
		cat:AddLine('text', L["No Transit Selected"])

	end

    tablet:SetHint(L["Click to cycle transport.|nAlt-Click to set up alarm"])
end

-- NOT using an AceOptions data table
local dewdrop = AceLibrary("Dewdrop-2.0")
function NauticusFu:OnMenuRequest(level, value, inTooltip)
	if inTooltip then return end

	dewdrop:AddLine(
		'text', L["Show only transports for your faction"],
		'arg1', self,
		'func', "ToggleFaction",
		'checked', self:IsFactionSpecific(),
		'tooltipTitle', L["Show only transports for your faction"],
		'tooltipText', L["Shows only neutral and transports specific to your faction."]
	)

	dewdrop:AddLine(
		'text', L["Show only transports in your current zone"],
		'arg1', self,
		'func', "ToggleZone",
		'checked', self:IsZoneSpecific(),
		'tooltipTitle', L["Show only transports in your current zone"],
		'tooltipText', L["Shows only transports in your current zone."]
	)

	dewdrop:AddLine(
		'text', L["Display using city aliases"],
		'arg1', self,
		'func', "ToggleAlias",
		'checked', self:IsAlias(),
		'tooltipTitle', L["Display using city aliases"],
		'tooltipText', L["Displays destinations as city aliases instead of zone names (e.g. Undercity instead of Tirisfal Glades)."]
	)

	dewdrop:AddSeparator()

	dewdrop:AddLine(
		'text', YELLOW.."Select None",
		'checked', (Nauticus.activeTransit == -1),
		'func', function() self:SetTransport(0) end
	)

	local textdesc, addtrans

	for i = 1, #(transports), 1 do
		addtrans = false

		if self:IsFactionSpecific() then
			if transports[i].faction == UnitFactionGroup("player") or
				transports[i].faction == "Neutral" then

				addtrans = true
			end
		else
			addtrans = true
		end

		if addtrans and self:IsZoneSpecific() then
			if not string.find(string.lower(transports[i].name),
				string.lower(GetRealZoneText())) then

				addtrans = false
			end
		end

		if addtrans then
			if self:IsAlias() then
				textdesc = transports[i].namealias
			else
				textdesc = transports[i].name
			end

			if Nauticus:GetKnownData(transports[i].label) ~= nil then
				textdesc = GREEN..textdesc
			end

			dewdrop:AddLine(
				'text', textdesc,
				'checked', (transports[i].label == Nauticus.activeTransit),
				'func', function(i) self:SetTransport(i) end,
				'arg1', i
			)
		end
	end

	dewdrop:AddSeparator()
end

function NauticusFu:OnEnable()
	self:ScheduleRepeatingEvent(self.OnUpdate, 1, self)
end

function NauticusFu:OnUpdate()
	self:Update()
end

function NauticusFu:OnTextUpdate()
	if Nauticus:IsAlarmSet() then
		self:SetIcon("Interface\\Icons\\INV_Misc_PocketWatch_02")
	else
		self:SetIcon(ARTWORKPATH..Nauticus.icon)
	end

	if Nauticus.tempTextCount > 0 then
		Nauticus.tempTextCount = Nauticus.tempTextCount -1
		return
	end

	self:SetText(Nauticus.lowestNameTime)
end

function NauticusFu:OnClick()

	if IsAltKeyDown() then
		if Nauticus.activeTransit ~= -1 then
			Nauticus:ToggleAlarm()

			-- set temporary button text so you know which one is currently selected
			if Nauticus:IsAlarmSet() then
				Nauticus.tempText = "Alarm "..RED.."ON"
			else
				Nauticus.tempText = "Alarm OFF"
			end
	
			Nauticus.tempTextCount = 2
			self:SetText(Nauticus.tempText)
			Nauticus.updateNow = true
		end

		return
	end

	local isNotEmpty, isFound, addtrans

	for i = 1, #(transports), 1 do
		addtrans = false

		if self:IsFactionSpecific() then
			if transports[i].faction == UnitFactionGroup("player") or
				transports[i].faction == "Neutral" then

				addtrans = true
			end
		else
			addtrans = true
		end

		if addtrans and self:IsZoneSpecific() then
			if not string.find(string.lower(transports[i].name),
				string.lower(GetRealZoneText())) then

				addtrans = false
			end
		end

		isNotEmpty = isNotEmpty or addtrans

		if not isFound then
			if transports[i].label == Nauticus.activeTransit then
				isFound = true
			end
		else
			if addtrans then
				addtrans = i
				break
			end
		end
	end

	if not isNotEmpty then
		addtrans = 0
	elseif type(addtrans) ~= "number" then
		addtrans = 1
	end

	Nauticus.activeTransit = transports[addtrans].label
	nautSavedVarsPC.config.activeTransit = Nauticus.activeTransit

	if self:IsAlias() then
		Nauticus.activeTransitName = transports[addtrans].namealias
	else
		Nauticus.activeTransitName = transports[addtrans].name
	end

	local color

	if Nauticus:GetKnownData(Nauticus.activeTransit) ~= nil then
		color = GREEN
	else
		color = RED
	end

	-- set temporary button text so you know which one is currently selected
	Nauticus.tempText = color..Nauticus.activeTransitName
	Nauticus.tempTextCount = 2
	self:SetText(Nauticus.tempText)
	Nauticus.updateNow = true
	Nauticus:TransportSelectSetNone()

end

function NauticusFu:SetTransport(id)
	local transport = transports[id]

	Nauticus.activeTransit = transport.label
	nautSavedVarsPC.config.activeTransit = Nauticus.activeTransit

	if id == 0 then
		Nauticus.activeTransitName = L["None Selected"]

	else
		if self:IsAlias() then
			Nauticus.activeTransitName = transport.namealias
		else
			Nauticus.activeTransitName = transport.name
		end

		Nauticus.tempText = Nauticus.activeTransitName
		Nauticus.tempTextCount = 2
		self:SetText(Nauticus.tempText)
		Nauticus.updateNow = true
	end

	Nauticus:TransportSelectSetNone()

end
