﻿
-- declard color codes for console messages
local RED     = "|cffff0000"
local GREEN   = "|cff00ff00"
local BLUE    = "|cff0000ff"
local MAGENTA = "|cffff00ff"
local YELLOW  = "|cffffff00"
local CYAN    = "|cff00ffff"
local WHITE   = "|cffffffff"
local ORANGE  = "|cffffba00"

local Nauticus = Nauticus

local L = AceLibrary("AceLocale-2.2"):new("Nauticus")

local rtts, transports, transitData =
	Nauticus.rtts, Nauticus.transports, Nauticus.transitData

-- local variables
local newVerAvail = false

function Nauticus:DoRequest()

	if self.requestVersion then
		self.requestVersion = false
		self:SendMessage("VER "..self.nautVersionNum)
	end

	if next(self.requestList) ~= nil then
		self:BroadcastTransportData()
		self.requestList = {}
	end

end

function Nauticus:BroadcastTransportData()
	local _,_,lag = GetNetStats()
	local now = GetTime()-lag/1000.0
	local to_send = {}

	for index, data in pairs(transports) do
		if data.label ~= -1 then
			local transit = data.label

			if self.requestList[transit] then
				local since = self:GetKnownData(transit)

				if since ~= nil then
					to_send[transit] = math.floor((now-since)*1000.0)/1000.0
				else
					to_send[transit] = "nil"
				end
			end
		end
	end

	if next(to_send) ~= nil then
		self:SendMessage("TELL "..self:TellToString(to_send))

		if self.debug then
			self:DebugMessage("tell our transports")
		end
	else
		if self.debug then
			self:DebugMessage("nothing to tell")
		end
	end

end

function Nauticus:SendMessage(msg)
	if self.dataChannel ~= "none" then
		SendChatMessage(msg, "CHANNEL", nil, GetChannelName(self.dataChannel) )
	end
end

function Nauticus:ReceiveMessage(msg, sender)
	local Args = self:GetArgs(msg, " ")
	if #(Args) <= 1 then return; end
	local command = Args[1]

	if command == "VER" then
		local clientversion = tonumber(Args[2])

		if clientversion > self.nautVersionNum then
			if not newVerAvail then
				newVerAvail = true
				DEFAULT_CHAT_FRAME:AddMessage(YELLOW.."Nauticus|r - "..RED..
					L["There is a new version of Nauticus available! Please visit http://wowui.incgamers.com, http://wow.curse.com or http://www.wowinterface.com."])
			end
		elseif clientversion < self.nautVersionNum then
			self.requestVersion = true
			self:ScheduleEvent("NAUT_REQUEST", self.DoRequest, 10+math.floor(math.random()*20), self)
			return
		end

		self.requestVersion = false

	-- KNOW/TRIG transit cycle
	elseif command == "KNOW" or command == "TRIG" then
		local _,_,lag = GetNetStats()
		local now = GetTime()-lag/1000.0
		local transit = Args[2]
		local value = tonumber(Args[3])

		self:DebugMessage("time "..value.." for "..transit.." received from "..sender)

		if self.lastcheck_timeout <= 10.0 and self.last_triggered == transit then
			self:DebugMessage("recently triggered")
			return
		end

		self:SetKnownData(transit, now-value)

	-- TELL transports
	elseif command == "TELL" then
		local _,_,lag = GetNetStats()
		local now = GetTime()-lag/1000.0
		local transports = self:StringToTell(Args[2])

		for transit, since in pairs(transports) do
			if since == "nil" then
				if self:GetKnownData(transit) ~= nil then
					self.requestList[transit] = true
					self:DebugMessage(sender.." says: tell "..transit..
						" nil - we have data for him")
				else
					self:DebugMessage(sender.." says: tell "..transit..
						" nil - nothing to tell either")
				end

			else
				if self:GetKnownData(transit) == nil then
					self:SetKnownData(transit, now - since)
					self:DebugMessage(sender.." says: tell "..transit.." "..since..
						" - ooh, he has data we don't")

				elseif tonumber(since) >= 0 then
					local clientage = math.floor( since / rtts[transit] )
					local age = math.floor( (now - self:GetKnownData(transit)) / rtts[transit] )

					if clientage < age then
						self:SetKnownData(transit, now - since)
						self.requestList[transit] = nil
						self:DebugMessage(sender.." says: tell "..transit.." "..since..
							" - ooh, he has better data")

					elseif clientage > age then
						self.requestList[transit] = true
						self:DebugMessage(sender.." says: tell "..transit.." "..since..
							" - aha, we have better data")

					else
						self.requestList[transit] = nil
						self:DebugMessage(sender.." says: tell "..transit.." "..since..
							" - his data is same as ours")

					end

				else
					self:DebugMessage(sender.." says: tell "..transit.." "..since..
						" - his data is BAD!")
				end
			end
		end

		self.tempText = RED.."Receiving Data.."
		self.tempTextCount = 2
		self:ScheduleEvent("NAUT_REQUEST", self.DoRequest, 10+math.floor(math.random()*20), self)
	end

end

function Nauticus:TellToString(transports)
	local trans_str = ""

	for transit, since in pairs(transports) do
		trans_str = trans_str..transit..":"..since..","
	end

	return string.sub(trans_str, 1, -2) -- remove the last comma
end

function Nauticus:StringToTell(transports)
	local Args = self:GetArgs(transports, ",")
	local trans_tab = {}

	for t = 1, #(Args), 1 do
		local Args_tmp = self:GetArgs(Args[t], ":")
		local transit, since = Args_tmp[1], Args_tmp[2]
		trans_tab[transit] = since
	end

	return trans_tab
end
