﻿
-- declard color codes for console messages
local RED     = "|cffff0000"
local GREEN   = "|cff00ff00"
local BLUE    = "|cff0000ff"
local MAGENTA = "|cffff00ff"
local YELLOW  = "|cffffff00"
local CYAN    = "|cff00ffff"
local WHITE   = "|cffffffff"
local ORANGE  = "|cffffba00"

-- constants
local PROX = 0.001
local PROX_HALF = PROX/2

Nauticus = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Nauticus")

local NautAstrolabe = DongleStub("Astrolabe-0.4")

local Nauticus = Nauticus
local rtts, platforms, triggers, transports, transitData

-- object variables
Nauticus.activeTransitName = ""
Nauticus.activeTransit = -1
Nauticus.lowestNameTime = "--"
Nauticus.icon = "NauticusLogo"
Nauticus.activeData = { [0] = 0, [1] = 0 }
Nauticus.tempText = ""
Nauticus.tempTextCount = 0
Nauticus.updateNow = false

Nauticus.dataChannel = "NauticSync"
Nauticus.nautVersion = "2.1.4"
Nauticus.nautVersionNum = 214

Nauticus.debug = false

-- local variables
local update_int = 0.5
local ctime_elapse = 0
local oldx, oldy = 0, 0
local lastcheck = 0
Nauticus.lastcheck_timeout = 10
Nauticus.last_triggered = nil
Nauticus.requestVersion = false
Nauticus.requestList = {}

local alarmSet = false
local alarmDinged = false
local alarmOffset = 20
local alarmCountdown = 0

-- calibration vars
--[[local isCalib = false
local cal_x, cal_y = 0, 0
local cal_old_x, cal_old_y = 0, 0
local cal_stopped = false]]


local Pre_ChatFrame_OnEvent

function Nauticus:OnInitialize()

	rtts, platforms, triggers, transports =
		self.rtts, self.platforms, self.triggers, self.transports,

	self:RegisterEvent("VARIABLES_LOADED")
	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("CHAT_MSG_CHANNEL")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	--self:RegisterEvent("CHAT_MSG_SYSTEM")
	--self:RegisterEvent("CHAT_MSG_CHANNEL_JOIN")
	self:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE")

	-- register slash command
	SLASH_NAUT1 = "/nauticus"
	SLASH_NAUT2 = "/naut"

	SlashCmdList["NAUT"] = function(msg)
		self:SlashCommandHandler(msg)
	end

	Pre_ChatFrame_OnEvent = ChatFrame_OnEvent
	ChatFrame_OnEvent = function(event)
		self:ChatFrame_OnEvent(event)
	end

	self:ScheduleRepeatingEvent(self.OnUpdate, update_int, self)

end

function Nauticus:ChatFrame_OnEvent(event)
	if event == "CHAT_MSG_CHANNEL" and self.dataChannel ~= "none" and
		string.lower(arg9) == string.lower(self.dataChannel) then

		-- silence
		if self.debug then
			return Pre_ChatFrame_OnEvent(event)
		end
	else
		return Pre_ChatFrame_OnEvent(event)
	end
end

function Nauticus:OnUpdate()
	local elapse = update_int

	if alarmDinged then
		alarmCountdown = alarmCountdown - elapse
		if 0 > alarmCountdown then
			alarmSet = false
			alarmDinged = false
			PlaySound("AuctionWindowClose")
		end
	end

	ctime_elapse = ctime_elapse + elapse
	if ctime_elapse > update_int or self.updateNow then
		ctime_elapse = 0

		if NautHeaderFrame.isMoving then self:UpdateUI() end

		if self.activeTransit ~= -1 and self:GetKnownData(self.activeTransit) ~= nil then
			local transit = self.activeTransit
			local cycle = self:CalcTripCycle(transit)
			local platform = self:CalcIndexByCycle(transit, cycle)
			local lowestTime = 500

			for index, data in pairs(platforms[transit]) do
				local platname

				if nautSavedVars.config.CityAlias then
					platname = data.alias
				else
					platname = data.name
				end

				getglobal("NautFramePlat"..(index+1).."Name"):SetText(platname)

				if data.index == platform then
					-- we're at a platform and waiting to depart
					local depart_time = self:CalcTripCycleTimeByIndex(transit, platform) - cycle

					if alarmSet and not alarmDinged and depart_time < alarmOffset then
						alarmDinged = true
						alarmCountdown = depart_time
						PlaySoundFile("Sound\\Spells\\PVPFlagTakenHorde.wav")
					end

					local formatted_depart_time

					if depart_time >= 60 then
						formatted_depart_time = format("%dm %02ds", depart_time/60,
							math.fmod(depart_time, 60) )
					else
						formatted_depart_time = format("%ds", depart_time)
					end

					local color

					if depart_time > 30 then
						color = YELLOW
					else
						color = RED
					end

					getglobal("NautFramePlat"..(index+1).."ArrivalDepature"):SetText(YELLOW..L["Departure"]..":")
					getglobal("NautFramePlat"..(index+1).."Time"):SetText(color..formatted_depart_time)
					self.activeData[index] = -depart_time

					lowestTime = -500
					self.lowestNameTime = data.ebv..color.." "..formatted_depart_time
					self.icon = "Departing"

				else
					local arrival_time = self:CalcTripCycleTimeByIndex(transit, data.index-1) - cycle

					if arrival_time < 0 then
						arrival_time = arrival_time + rtts[transit]
					end

					local formatted_arrival_time

					if arrival_time >= 60 then
						formatted_arrival_time = format("%dm %02ds", arrival_time/60,
							math.fmod(arrival_time, 60) )
					else
						formatted_arrival_time = format("%ds", arrival_time)
					end

					getglobal("NautFramePlat"..(index+1).."ArrivalDepature"):SetText(YELLOW..L["Arrival"]..":")
					getglobal("NautFramePlat"..(index+1).."Time"):SetText(GREEN..formatted_arrival_time)
					self.activeData[index] = arrival_time

					if arrival_time < lowestTime then
						lowestTime = arrival_time
						self.lowestNameTime = data.ebv.." "..GREEN..formatted_arrival_time
						self.icon = "Transit"
					end

				end

			end

			if self.updateNow and NauticusFu ~= nil then
				NauticusFu:Update()
			end

		end

		self.updateNow = false
	end

	lastcheck = lastcheck + elapse
	if 0.2 > lastcheck then return; end
	lastcheck = 0

	if IsInInstance() or IsSwimming() then return end

	self.lastcheck_timeout = self.lastcheck_timeout + elapse
	if 10.0 > self.lastcheck_timeout then return; end

	local c, z, x, y = NautAstrolabe:GetCurrentPlayerPosition()
	if not x then return; end
	x, y = NautAstrolabe:TranslateWorldMapPosition(c, z, x, y, 0, 0)
	if not x then return; end

	-- have we moved by at least half the proximity range?
	if abs(x - oldx) > PROX_HALF or abs(y - oldy) > PROX_HALF then
		oldx, oldy = x, y

		--check X/Y coords against all triggers for all transports
		for t = 1, #(transports), 1 do
			local transit = transports[t].label

			for i, index in pairs(triggers[transit]) do

				if abs(x - transitData[transit].x[index]) <= PROX and
					abs(y - transitData[transit].y[index]) <= PROX then

					self:SetKnownTime(transit, index)
					return
				end

			end
		end
	end

end

--TODO: store last indexes of all transports?
local lastIndex = 1
function Nauticus:CalcIndexByCycle(transit, cycle)
	local offsets = transitData[transit].offset

	if lastIndex > #(offsets) or
		(lastIndex > 1 and offsets[lastIndex-1] > cycle) then
			lastIndex = 1; end

	for i = lastIndex, #(offsets) do
		if offsets[i] > cycle then
			lastIndex = i
			return i
		end
	end

	--should never get here but just in case of math errors?
	if self.debug then self:DebugMessage("math error?") end
	return 1
end

function Nauticus:CalcTripCycleTimeByIndex(transit, index)
	if index == 0 then return 0 end
	return transitData[transit].offset[index]
end

function Nauticus:CalcTripCycle(transit)
	return math.fmod(GetTime()-self:GetKnownData(transit), rtts[transit])
end

function Nauticus:SetKnownTime(transit, index)

	self.lastcheck_timeout = 0
	self.last_triggered = transit

	local now = GetTime()
	local the_time = time()
	local sum_time = self:CalcTripCycleTimeByIndex(transit, index)

	if self.debug then
		if self:GetKnownData(transit) ~= nil then
			local oldCycle = self:CalcTripCycle(transit)
			self:DebugMessage(transit..", cycle time: "..sum_time.." ; old: "
				..format("%0.3f", oldCycle).." ; diff: "..format("%0.3f", oldCycle-sum_time)
				.." ; drift: "..format("%0.6f",
					(oldCycle-sum_time)/((now-sum_time-self:GetKnownData(transit))/rtts[transit]) )
				)
		else
			self:DebugMessage(transit..", cycle time: "..sum_time)
		end
	end

	self:SetKnownData(transit, now-sum_time)
	nautSavedVars.uptime = now
	nautSavedVars.timestamp = the_time

	local _,_,lag = GetNetStats()

	self:SendMessage("TRIG "..transit.." "..format("%0.3f", sum_time-lag/1000.0))

end

function Nauticus:InitialiseConfig()

	-- initialise saved variables
	if nautSavedVars == nil then
		nautSavedVars = {}
	end

	local savedVars = nautSavedVars

	if savedVars.knownTimes == nil then
		savedVars.knownTimes = {}
	end

	if nautSavedVarsPC == nil then
		nautSavedVarsPC = {}
	end

	local savedVarsPC = nautSavedVarsPC

	if savedVars.config == nil then
		savedVars.config = {}
		savedVars.config.ZoneGUI = false
		savedVars.config.FactionSpecific = true
		savedVars.config.ZoneSpecific = false
		savedVars.config.CityAlias = true
	end

	local cfg = savedVars.config

	if savedVarsPC.config == nil then
		savedVarsPC.config = {}
	end

	local cfgPC = savedVarsPC.config

	if cfg.dataChannel == nil then
		cfg.dataChannel = self.dataChannel
	end

	self.dataChannel = cfg.dataChannel

	if cfg.debug == nil then
		cfg.debug = false
	end

	self.debug = cfg.debug

	if cfg.alarmOffset == nil then
		cfg.alarmOffset = alarmOffset
	end

	alarmOffset = cfg.alarmOffset

	if cfgPC.ShowGUI == nil then
		cfgPC.ShowGUI = true
	end

	if cfgPC.ShowLowerGUI == nil then
		cfgPC.ShowLowerGUI = true
	end

	if cfgPC.activeTransit == nil then
		cfgPC.activeTransit = -1
	end

	self.activeTransit = cfgPC.activeTransit
	self:InitialiseUI()

	local now = GetTime()
	local the_time = time()

	if savedVars.uptime == nil then
		-- clear known times
		savedVars.knownTimes = {}

	else
		-- calculate potential drift time in ms between sessions
		local drift = (the_time-now)-(savedVars.timestamp-savedVars.uptime)

		self:DebugMessage("boot drift: "..drift)

		-- if more than 3 mins drift, that means reboot occured. we need to adjust ms timers
		if math.abs(drift) > 180 then
			-- adjust all available transport times by drift
			for transport, times in pairs(savedVars.knownTimes) do
				self:SetKnownData(transport, self:GetKnownData(transport)-drift)
			end

			self:DebugMessage("reboot must have occured")
		end

	end

	-- record uptime and 'when' (relative to local system clock) this was made
	savedVars.uptime = now
	savedVars.timestamp = the_time

end

function Nauticus:VARIABLES_LOADED()
	self:InitialiseConfig()
end

function Nauticus:PLAYER_LOGIN()

	-- unpack transport data
	local packedData = self.packedData
	local args = {}
	local j, oldX, oldY, oldOffset, transit, transit_data

	self.transitData = {}
	transitData = self.transitData

	for t = 1, #(transports), 1 do
		oldX, oldY, oldOffset = 0, 0, 0

		transit = transports[t].label
		transitData[transit] = { ['x'] = {}, ['y'] = {}, ['offset'] = {}, }
		transit_data = transitData[transit]

		for i = 1, #(packedData[transit]) do
			j = 0
			-- search for seperators in the string and return the separated data
			for value in string.gmatch(packedData[transit][i], "[^:]+") do
				j = j + 1; args[j] = value
			end

			transit_data.x[i] = args[1]+oldX
			transit_data.y[i] = args[2]+oldY
			transit_data.offset[i] = args[3]+60+oldOffset

			oldX, oldY, oldOffset =
				transit_data.x[i],
				transit_data.y[i],
				transit_data.offset[i]-60
		end

		transit_data.offset[#(packedData[transit])] = rtts[transit]

		--self:DebugMessage(transports[t].label..": "..transit_data.offset[#(packedData[transit])])
	end

	self.packedData = nil -- free some memory

	if not nautSavedVarsPC.config.ShowLowerGUI then
		-- initialise anchor
		local right = NautHeaderFrame:GetRight()
		local top = NautHeaderFrame:GetTop()

		if top ~= nil and right ~= nil then
			NautHeaderFrame:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", right-146, top)
		end
	end

	self:TransportSelectSetNone()
	self:UpdateUI()

end

-- if we joined a channel
function Nauticus:CHAT_MSG_CHANNEL_NOTICE(noticeType, _, _, numAndName, _, _, _, num, channel)
	-- automatically join data channel
	if self.dataChannel ~= "none" and GetChannelName(self.dataChannel) == 0 then
		JoinChannelByName(self.dataChannel)
	end

	if channel == self.dataChannel then
		for index, data in pairs(transports) do
			if data.label ~= -1 then
				self.requestList[data.label] = true
			end
		end

		self.requestVersion = true
		self:ScheduleEvent("NAUT_REQUEST", self.DoRequest, 10+math.floor(math.random()*20), self)
	end
end

function Nauticus:CHAT_MSG_CHANNEL(msg, sender, _, _, _, _, _, _, channel)
	if channel == self.dataChannel and sender ~= UnitName("player") then
		self:ReceiveMessage(msg, sender)
	end
end

function Nauticus:ZONE_CHANGED_NEW_AREA()
	-- show GUI when zone change contains a transport
	if not nautSavedVars.config.ZoneGUI then return end

	-- open GUI if entering zone with transport
	for index, zone_data in pairs(transports) do
		if zone_data.label ~= -1 then
			local plat_data = platforms[zone_data.label]
			if plat_data ~= nil then
				for index, trans_data in pairs(plat_data) do
					if trans_data.name == GetRealZoneText() then
						nautSavedVarsPC.config.ShowLowerGUI = true
						nautSavedVarsPC.config.ShowGUI = true
						self:UpdateUI()
					end
				end
			end
		end
	end
end

function Nauticus:ToggleAlarm()
	alarmSet = not alarmSet
	if not alarmSet then alarmDinged = false end
	local is; if alarmSet then is = RED.."ON|r" else is = GREEN.."OFF|r" end
	DEFAULT_CHAT_FRAME:AddMessage(YELLOW.."Nauticus|r - "..WHITE.."Alarm is now: "..is)
	PlaySound("AuctionWindowOpen")
end

function Nauticus:IsAlarmSet()
	return alarmSet or alarmDinged
end

function Nauticus:SlashCommandHandler(msg)
	local msgArgs = self:GetArgs(msg, " ")
	local numArgs = #(msgArgs)

	if msgArgs[1] then msgArgs[1] = string.lower(msgArgs[1]); end

	if numArgs == 0 then
		DEFAULT_CHAT_FRAME:AddMessage(YELLOW.."Nauticus|r - "..WHITE.."Display GUI")
		nautSavedVarsPC.config.ShowGUI = true
		self:UpdateUI()

	elseif numArgs == 1 then

		if msgArgs[1] == "reset" then
			DEFAULT_CHAT_FRAME:AddMessage(RED.."Nauticus - Session Reset!")
			nautSavedVars.knownTimes = {}

		elseif msgArgs[1] == "debug" then
			nautSavedVars.config.debug = not nautSavedVars.config.debug
			self.debug = nautSavedVars.config.debug

			local is; if self.debug then is = RED.."ON" else is = GREEN.."OFF" end
			DEFAULT_CHAT_FRAME:AddMessage(YELLOW.."Nauticus|r - "..WHITE.."Debug mode now: "..is)

		--[[elseif msgArgs[1] == "calib" then
			if isCalib then
				Naut_DataTrackerFrame:SetScript("OnUpdate", nil)
			else
				local c, z; c, z, cal_x, cal_y = NautAstrolabe:GetCurrentPlayerPosition()
				if cal_x then cal_x, cal_y = NautAstrolabe:TranslateWorldMapPosition(c, z, cal_x, cal_y, 0, 0); end
				cal_old_x, cal_old_y = cal_x, cal_y
				cal_stopped = true
				Naut_DataTrackerFrame:SetScript("OnUpdate",
					function (this, elapse) self:OnFrameUpdate(this, elapse) end)
			end
			isCalib = not isCalib
			local is; if isCalib then is = RED.."ON" else is = GREEN.."OFF" end
			DEFAULT_CHAT_FRAME:AddMessage(YELLOW.."Nauticus|r - "..WHITE.."Calibration now: "..is)]]

		end

	elseif numArgs == 2 then

		if msgArgs[1] == "channel" then
			if self.dataChannel ~= "none" then
				LeaveChannelByName(self.dataChannel)
			end

			nautSavedVars.config.dataChannel = msgArgs[2]
			self.dataChannel = nautSavedVars.config.dataChannel

			if self.dataChannel ~= "none" then
				JoinChannelByName(self.dataChannel)
			end

			DEFAULT_CHAT_FRAME:AddMessage(YELLOW.."Nauticus|r"..WHITE..
				" - Sync channel set to '"..self.dataChannel.."'")

		elseif msgArgs[1] == "alarm" then
			local offset = tonumber(msgArgs[2])

			if offset ~= nil then
				nautSavedVars.config.alarmOffset = offset
				alarmOffset = nautSavedVars.config.alarmOffset

				DEFAULT_CHAT_FRAME:AddMessage(YELLOW.."Nauticus|r"..WHITE..
					" - Alarm delay set to: "..offset)
			end

		end

	end

end

-- calibration code executed each frame
--[[function Nauticus:OnFrameUpdate(elapse)

	local now = GetTime()
	cal_old_x, cal_old_y = cal_x, cal_y

	local c, z; c, z, cal_x, cal_y = NautAstrolabe:GetCurrentPlayerPosition()
	if not cal_x then return; end
	cal_x, cal_y = NautAstrolabe:TranslateWorldMapPosition(c, z, cal_x, cal_y, 0, 0)
	if not cal_x or not cal_old_x then return; end

	if cal_stopped and (cal_x ~= cal_old_x or cal_y ~= cal_old_y) then
		self:DebugMessage("Started @ "..now)
		cal_stopped = false
	end
end]]

function Nauticus:GetKnownData(transport)
	return nautSavedVars.knownTimes[transport]
end

function Nauticus:SetKnownData(transport, since)
	nautSavedVars.knownTimes[transport] = since
end

-- extract key/value from message
function Nauticus:GetArgs(message, separator)

	local args = {}
	local i = 0

	-- search for seperators in the string and return the separated data
	for value in string.gmatch(message, "[^"..separator.."]+") do
		i = i + 1
		args[i] = value
	end

	return args
end

function Nauticus:DebugMessage(msg)
	if self.debug then
		ChatFrame4:AddMessage("[Naut]: "..msg)
	end
end
