﻿
-- declard color codes for console messages
local RED     = "|cffff0000"
local GREEN   = "|cff00ff00"
local BLUE    = "|cff0000ff"
local MAGENTA = "|cffff00ff"
local YELLOW  = "|cffffff00"
local CYAN    = "|cff00ffff"
local WHITE   = "|cffffffff"
local ORANGE  = "|cffffba00"

local Nauticus = Nauticus

local L = AceLibrary("AceLocale-2.2"):new("Nauticus")

local platforms, transports, transitData =
	Nauticus.platforms, Nauticus.transports, Nauticus.transitData

function Nauticus:TransportSelectInitialise(level)

	if level == 1 then
		local info, textdesc, addtrans

		for i = 0, #(transports), 1 do
			addtrans = false

			if nautSavedVars.config.FactionSpecific then
				local faction = UnitFactionGroup("player")

				if transports[i].faction == faction or
					transports[i].faction == "Neutral" then

					addtrans = true
				end
			else
				addtrans = true
			end

			if addtrans and nautSavedVars.config.ZoneSpecific then
				local zonestr = string.lower(transports[i].name)
				local czonestr = string.lower(GetRealZoneText())

				if not string.find(zonestr, czonestr) then
					addtrans = false
				end
			end

			if addtrans or transports[i].faction == -1 then
				if nautSavedVars.config.CityAlias then
					textdesc = transports[i].namealias
				else
					textdesc = transports[i].name
				end

				if self:GetKnownData(transports[i].label) then
					textdesc = GREEN..textdesc
				end

				info = UIDropDownMenu_CreateInfo()
				info.text = textdesc
				info.value = i
				if transports[i].label == self.activeTransit then info.checked = true; end
				info.func = function() Nauticus:TransportSelect_OnClick() end

				UIDropDownMenu_AddButton(info)
			end
		end
	end

end

function Nauticus:TransportSelect_OnClick()
	local transport = transports[this.value]

	Nauticus.activeTransit = transport.label
	nautSavedVarsPC.config.activeTransit = Nauticus.activeTransit

	if nautSavedVars.config.CityAlias then
		Nauticus.activeTransitName = transport.namealias
	else
		Nauticus.activeTransitName = transport.name
	end

	Nauticus:TransportSelectSetNone()
end

function Nauticus:TransportSelectSetNone()

	if self.activeTransit == -1 then
		for index = 1, 2 do
			getglobal("NautFramePlat"..(index).."Name"):SetText(L["None Selected"])
			getglobal("NautFramePlat"..(index).."ArrivalDepature"):SetText(L["Not Available"])
			getglobal("NautFramePlat"..(index).."Time"):SetText(L["N/A"])
		end

		self.lowestNameTime = "--"
		self.icon = "NauticusLogo"

	elseif self:GetKnownData(self.activeTransit) == nil then
		local transit = self.activeTransit

		for index, data in pairs(platforms[transit]) do
			local platname

			if nautSavedVars.config.CityAlias then
				platname = data.alias
			else
				platname = data.name
			end

			getglobal("NautFramePlat"..(index+1).."Name"):SetText(platname)
			getglobal("NautFramePlat"..(index+1).."ArrivalDepature"):SetText(L["Not Available"])
			getglobal("NautFramePlat"..(index+1).."Time"):SetText(L["N/A"])
		end

		self.lowestNameTime = L["N/A"]
		self.icon = "NauticusLogo"

	else
		lastIndex = 1

	end

end

function Nauticus:Minimise_OnClick()
	nautSavedVarsPC.config.ShowLowerGUI = not nautSavedVarsPC.config.ShowLowerGUI
	self:UpdateUI()
end

function Nauticus:UpdateUI()

	if nautSavedVarsPC.config.ShowGUI then
		NautHeaderFrame:Show()
	else
		NautHeaderFrame:Hide()
		return
	end

	if nautSavedVarsPC.config.ShowLowerGUI then
		NautHeaderFrame:SetWidth(195)
		NautHeaderFrameAddonName:Show()
		NautHeaderFrameOptionsButton:Show()
		self:CancelScheduledEvent("NAUT_HIDE_NautFrame")
		NautFrame:SetAlpha(1)
		NautFrame:Show()
		NautHeaderFrameMinimiseButton:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-UP")
		NautHeaderFrameMinimiseButton:SetPushedTexture("Interface\\Buttons\\UI-MinusButton-DOWN")

		local _, centre = NautHeaderFrame:GetCenter()
		NautFrame:ClearAllPoints()

		if centre < (GetScreenHeight()/2) then
			NautFrame:SetPoint("BOTTOM", NautHeaderFrame, "TOP", 0, -5)
		else
			NautFrame:SetPoint("TOP", NautHeaderFrame, "BOTTOM", 0, 5)
		end

	else
		if NautFrame:IsVisible() then
			self:TransportSelectButton_Fade("NautFrame", 0.85, function() self:UpdateUI() end)
			return
		end

		NautHeaderFrameMinimiseButton:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-UP")
		NautHeaderFrameMinimiseButton:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-DOWN")
		NautHeaderFrameOptionsButton:Hide()
		NautHeaderFrameAddonName:Hide()

		-- initialise anchor
		local right = NautHeaderFrame:GetRight()
		local top = NautHeaderFrame:GetTop()

		if top ~= nil and right ~= nil then
			NautHeaderFrame:ClearAllPoints()
			NautHeaderFrame:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", right, top)
		end

		NautHeaderFrame:SetWidth(64)
	end

end

function Nauticus:Close_OnClick()
	nautSavedVarsPC.config.ShowGUI = false
	self:UpdateUI()

	DEFAULT_CHAT_FRAME:AddMessage(YELLOW.. "Nauticus|r - "..WHITE..
		L["GUI closed. Type /nauticus or /naut to show again."])
end

function Nauticus:Options_OnClick()

	if not NautOptionsFrame:IsVisible() then
		NautOptionsFrame:Show()
	else
		NautOptionsFrame:Hide()
	end

end

function Nauticus:OptionsSave_OnClick()

	nautSavedVars.config.ZoneGUI = NautOptionsFrameOptZoneGUI:GetChecked()
	nautSavedVars.config.FactionSpecific = NautOptionsFrameOptFactionSpecific:GetChecked()
	nautSavedVars.config.ZoneSpecific = NautOptionsFrameOptZoneSpecific:GetChecked()
	nautSavedVars.config.CityAlias = NautOptionsFrameOptCityAlias:GetChecked()

	NautOptionsFrame:Hide()

end

function Nauticus:OptionsClose_OnClick()

	if nautSavedVars.config.ZoneGUI then
		NautOptionsFrameOptZoneGUI:SetChecked()
	end
	if nautSavedVars.config.FactionSpecific then
		NautOptionsFrameOptFactionSpecific:SetChecked()
	end
	if nautSavedVars.config.ZoneSpecific then
		NautOptionsFrameOptZoneSpecific:SetChecked()
	end
	if nautSavedVars.config.CityAlias then
		NautOptionsFrameOptCityAlias:SetChecked()
	end

	NautOptionsFrame:Hide()

end

function Nauticus:InitialiseUI()

	local cfg = nautSavedVars.config

	-- set GUI options
	if cfg.ZoneGUI then
		NautOptionsFrameOptZoneGUI:SetChecked()
	end
	if cfg.FactionSpecific then
		NautOptionsFrameOptFactionSpecific:SetChecked()
	end
	if cfg.ZoneSpecific then
		NautOptionsFrameOptZoneSpecific:SetChecked()
	end
	if cfg.CityAlias then
		NautOptionsFrameOptCityAlias:SetChecked()
	end

	for i = 0, #(transports), 1 do
		if transports[i].label == self.activeTransit then
			if cfg.CityAlias then
				self.activeTransitName = transports[i].namealias
			else
				self.activeTransitName = transports[i].name
			end

			break
		end
	end

	NautHeaderFrameAddonName:SetText("Nauticus v"..self.nautVersion)

	if (GetLocale() == "deDE") then
		NautOptionsFrame:SetWidth(365)
		NautOptionsFrameCloseOptionsButton:SetWidth(75)
		NautOptionsFrameSaveOptionsButton:SetWidth(75)
	end

	NautOptionsFrameOptionsTitle:SetText(L["Nauticus Options"])
	NautOptionsFrameOptZoneGUIText:SetText(L["Show GUI when zone change contains a transport"])
	NautOptionsFrameOptFactionSpecificText:SetText(L["Show only transports for your faction"])
	NautOptionsFrameOptZoneSpecificText:SetText(L["Show only transports in your current zone"])
	NautOptionsFrameOptCityAliasText:SetText(L["Display using city aliases"])

	NautOptionsFrameCloseOptionsButtonText:SetText(L["Close"])
	NautOptionsFrameSaveOptionsButtonText:SetText(L["Save"])

	DEFAULT_CHAT_FRAME:AddMessage(YELLOW.. string.format(L["Nauticus v%s loaded. Type /nauticus or /naut to show GUI."], self.nautVersion))

end

-- recursively fade the frame
-- if fade is nil, first pause 1.5sec then fade over 0.5sec @ ~13.33fps
-- callback is function to call when frame eventually hidden (optional)
function Nauticus:TransportSelectButton_Fade(frame_name, fade, callback)
	frame = getglobal(frame_name)

	if not fade then
		self:ScheduleEvent("NAUT_HIDE_"..frame_name,
			self.TransportSelectButton_Fade, 1.5, self, frame_name, 0.85, callback)
	else
		if fade < 0 then
			frame:Hide()
			if callback then callback(); end
		else
			frame:SetAlpha(fade)

			self:ScheduleEvent("NAUT_HIDE_"..frame_name,
				self.TransportSelectButton_Fade, 0.075, self, frame_name, fade-0.15, callback)
		end
	end
end

function Nauticus:ShowTooltip(text)
	self:ScheduleEvent("NAUT_SHOW_TOOLTIP", function(text, frame)
		GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
		GameTooltip:SetText(L[text])
	end, 1, text, this)
end

function Nauticus:GameTooltip_Hide()
	self:CancelScheduledEvent("NAUT_SHOW_TOOLTIP")
	GameTooltip:Hide()
end
